#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

char *addStringQuotes(int type)
{
   if(
      type == FIELD_TYPE_DATE       || type == FIELD_TYPE_TIME   ||
      type == FIELD_TYPE_DATETIME   || type == FIELD_TYPE_STRING ||
      type == FIELD_TYPE_VAR_STRING
      )
     return "'";
   else
     return "";
}

void updateRecordAction(GtkButton *button, gpointer rec)
{
   Record *r = (Record *)rec;
   char *query;
   int y;
   int size = 100; /* safe start */
   
   ldebug(2,"updateRecordAction entered\n");
   
   /* get approximate querylength: account for quotes,spaces and equalsigns */
   for(y=0;y<r->numflds;y++)
     size += 2 * (strlen(r->fields[y].name) + r->fields[y].length + 10);
   query =(char *)g_malloc(size * sizeof(char));
   
   snprintf(query, size - 1, UPDATE , r->table);
   
   /* new values : */
   for(y=0;y<r->numflds;y++)
   {
      /* text entry can currently be larger than fieldlength: we 
       need strncat to truncate and thus avoid buffer overflows */
   
      strcat (query, r->fields[y].name);
      strcat (query, EQUAL);
      strcat (query, addStringQuotes(r->fields[y].type));
      strncat(query, gtk_entry_get_text(r->entries[y]), r->fields[y].length);
      strcat (query, addStringQuotes(r->fields[y].type));
      
      if(y< r->numflds - 1)
	strcat(query, ",");
   }
   strcat(query, WHERE);
   
   /* old values: */
   for(y=0;y<r->numflds;y++)
   {
      strcat (query, r->fields[y].name);
      strcat (query, EQUAL);
      strcat (query, addStringQuotes(r->fields[y].type));
      strcat (query, r->contents[y]);
      strcat (query, addStringQuotes(r->fields[y].type));
      
      if(y< r->numflds - 1)
	strcat(query, AND);
   }
   
   ldebug(3,"query space: %d, actual querylength : %d\n",size, strlen(query));
   
   if( executeQuery(r->dbsock, query) == 0)
   {  
      writeLog("Record update successful\n");
      refreshRecordTab();
   }
   else
     writeLog("Record update failed\n");
   
   defaultAction(button, NULL); /* close dialog window */
   destroyRecordWindow(r); /* close record window */
   g_free(query);
}

void deleteRecordAction(GtkButton *button, gpointer rec)
{
   Record *r = (Record *)rec;
   char *query;
   int y;
   int size = 100; /* safe start */
   
   ldebug(2,"deleteRecordAction entered\n");
   
   /* get approximate querylength: account for quotes,spaces and equalsigns */
   for(y=0;y<r->numflds;y++)
     size += strlen(r->fields[y].name) + r->fields[y].length + 10;
   query =(char *)g_malloc(size * sizeof(char));
   
   snprintf(query, size - 1, DELETE WHERE , r->table);
   
   for(y=0;y<r->numflds;y++)
   {
      strcat (query, r->fields[y].name);
      strcat (query, EQUAL);
      strcat (query, addStringQuotes(r->fields[y].type));
      strcat (query, r->contents[y]);
      strcat (query, addStringQuotes(r->fields[y].type));
      
      if(y< r->numflds - 1)
	strcat(query,AND);
   }
   
   ldebug(3,"query space: %d, actual querylength : %d\n",size, strlen(query));

   if( executeQuery(r->dbsock, query) == 0)
   {
      writeLog("Record deletion successful\n");
      refreshRecordTab();
    }
   else
     writeLog("Record deletion failed\n");
   
   defaultAction(button, NULL); /* close dialog window */
   destroyRecordWindow(r); /* close record window */
   g_free(query);
}

void insertRecordAction(GtkButton *button, gpointer rec)
{
   Record *r = (Record *)rec;
   char *query;
   int y;
   int size=100; /* safe start */
   
   ldebug(2,"insertRecordAction entered\n");
   
   /* get approximate querylength: account for quotes,spaces and equalsigns */
   for(y=0;y<r->numflds;y++)
     size += strlen(r->fields[y].name) + r->fields[y].length + 10;
   query =(char *)g_malloc(size * sizeof(char));
   
   snprintf(query,size -1,INSERT,r->table);
   
   for(y=0;y<r->numflds;y++)
   {
      /* text entry can currently be larger than fieldlength: we 
       need strncat to truncate and thus avoid buffer overflows */
   
      strcat (query, addStringQuotes(r->fields[y].type));
      strncat(query, gtk_entry_get_text(r->entries[y]), r->fields[y].length);
      strcat (query, addStringQuotes(r->fields[y].type));
      
      if(y< r->numflds - 1)
	strcat(query,",");
   }
   strcat(query,")");
   
   ldebug(3,"query space: %d, actual querylength : %d\n",size, strlen(query));

   if( executeQuery(r->dbsock, query) == 0)
   {
      writeLog("Record insertion successful\n");
      refreshRecordTab();
   }else
     writeLog("Record insertion failed\n");
   
   defaultAction(button, NULL); /* close dialog window */
//   destroyRecordWindow(r); /* close record window : */ /* keep it for multiple inserts */ 
   g_free(query);
}

