#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

/* First the callbacks for MainWindow : */

/* Menus in main window : */

void on_file_loadquery_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   Params *p = getParams();
   newFileSelector("query.saved",&(p->selectedname),GTK_FILE_CHOOSER_ACTION_OPEN);
   loadQueryAction( NULL, p);
}

void on_file_savequery_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   Params *p = getParams();
   newFileSelector("query.saved",&(p->selectedname),GTK_FILE_CHOOSER_ACTION_SAVE);
   saveQueryAction( NULL, p);
}

void on_file_saverecords_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   Params *p = getParams();
   TabInfo *ti = &p->tabinfo[gtk_notebook_get_current_page
			 (GTK_NOTEBOOK(p->notebook))];
   if(ti != NULL) /* correct ? */
     newFileSelector("records.out",&(ti->selectedname),GTK_FILE_CHOOSER_ACTION_SAVE);
   saveTabAction( NULL, ti);
}

void on_file_savelog_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   Params *p = getParams();
   newFileSelector("gequel.log",&(p->selectedname),GTK_FILE_CHOOSER_ACTION_SAVE);
   saveLogAction( NULL, p);
}

void on_file_print_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   showMessage("Sorry: not implemented yet");
}

void on_file_exit_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   ldebug(2,"on_exit_activate called\n");
   gtk_main_quit();
}

void on_edit_insert_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   newInsertWindow();
}

void on_edit_delete_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   Params *p = getParams();
   TabInfo *ti = &p->tabinfo[RECORDSTAB];
   
   if(ti == NULL || ti->selectedrow == -1)
   {
      showMessage("Please open browser and/or select a record to delete !");
      return;
   }
   
   /* the selected row is already stored in the record_selected callback */
   on_recorddelete_clicked(NULL, newRecordstruct(ti)); /* call callback */
}

void on_edit_modify_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   Params *p = getParams();
   TabInfo *ti = &p->tabinfo[RECORDSTAB];
   /* the selected row is stored in the record_selected callback */
   newModifyWindow(ti);
}

void on_edit_copy_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   Params *p = getParams();
   TabInfo *ti = &p->tabinfo[RECORDSTAB];
   /* the selected row is stored in the record_selected callback */
   newCopyMainWindow(ti);
}

void on_settings_confirm_actions_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   GtkCheckMenuItem *confirm_actions = GTK_CHECK_MENU_ITEM(menuitem);
   if(confirm_actions->active == 0)
     showMessage("Warning : all record actions will be immediately "
		 "executed skipping any confirmation !");
}

void on_settings_show_grid_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   GtkCheckMenuItem *show_grid = GTK_CHECK_MENU_ITEM(menuitem);
   Params *p = getParams();
   GtkTreeViewGridLines state=GTK_TREE_VIEW_GRID_LINES_BOTH;
   int x;
   
   if(show_grid->active == 0)
     state = GTK_TREE_VIEW_GRID_LINES_NONE;
   for(x=0;x<p->nrtabs;x++)
     gtk_tree_view_set_grid_lines (p->tabinfo[x].view,state);   
}

void on_help_contents_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   showMessage("For more information about how to use this\n"
	       "program you can for instance take a look in\n"
	       "the README file which comes with the\n"
	       "distribution of Gequel.");
}

void on_help_about_activate(GtkMenuItem *menuitem, gpointer user_data)
{
   showMessage("GEQUEL" GEQUELVERSION ": MySQL database\n"
	       "manipulation GUI written by Kees Lemmens\n"
	       "TU Delft, March 2003\n\n"
	       "Similar to the XForms based program\n "
	       "'xsqlmenu' by the same author in 1996\n\n"
	       "GNU Public License (but use on\n"
	       "Micro$oft systems is strongly disadvised ;-)");
}

/* Tabs in main window : */

void on_notebooktab_switched(GtkNotebook *notebook, GtkNotebookPage *page,
			    int pagenum, gpointer user_data)
{
   Params *p =  getParams();
   
   ldebug(2,"on_notebooktab_switched called\n");
   
   if(p->tabinfo[pagenum].refresh == FALSE)
    return; // this is to avoid endless loops !
   
   switch(pagenum)
   {
    case RECORDSTAB: showRecords(); break;
    case FIELDSTAB:  showFields();  break;
    case SEARCHTAB:  if( p->tabinfo[pagenum].inuse == FALSE) // only if no previous search exists
	{newSearchWindow();}        break;
      
    default : break;
   }
   p->tabinfo[pagenum].refresh = FALSE;
}

/* Buttons in main window : */

void on_showrecords_clicked(GtkButton *button, gpointer user_data)
{
   forceRefreshRecordTab();  // calls notebook callback
}
   
void on_showfields_clicked(GtkButton *button, gpointer user_data)
{
  refreshFieldTab(); // calls notebook callback
}

void on_insert_clicked(GtkButton *button, gpointer user_data)
{
   on_edit_insert_activate(NULL, NULL); /* call other callback */
}

void on_delete_clicked(GtkButton *button, gpointer user_data)
{
   on_edit_delete_activate(NULL, NULL); /* call other callback */
}

void on_modify_clicked(GtkButton *button, gpointer user_data)
{
   on_edit_modify_activate(NULL, NULL); /* call other callback */
}

void on_copy_clicked(GtkButton *button, gpointer user_data)
{
   on_edit_copy_activate(NULL, NULL); /* call other callback */
}

void on_search_clicked(GtkButton *button, gpointer user_data)
{
   newSearchWindow();
}

void on_createtable_clicked(GtkButton *button, gpointer user_data)
{
   showMessage("Sorry: not implemented yet");
}

void on_edittable_clicked(GtkButton *button, gpointer user_data)
{
   showMessage("Sorry: not implemented yet");
}

void on_query_clicked(GtkButton *button, gpointer user_data)
{
   Params *p = getParams();
   GtkTextIter    txtstart;
   GtkTextIter    txtend;

   ldebug(2,"on_query_clicked called\n");
   
   gtk_text_buffer_get_start_iter (p->querybuffer, &txtstart);
   gtk_text_buffer_get_end_iter   (p->querybuffer, &txtend);
   showQuery(gtk_text_buffer_get_text(p->querybuffer,&txtstart,&txtend,FALSE));
}

/* Text entries in main window : */

void on_server_activate(GtkEditable *editable, gpointer user_data)
{
   Params *p = getParams();
   setNewServer((gchar *)gtk_entry_get_text(GTK_ENTRY(editable)),p->socketname);
}

void usage(char *name)
{
   fprintf(stderr,"\nUsage  : %s [options] [<database> [<table>]]\n",name);
   fprintf(stderr,  "Options: -h <host>   : set remote database server\n");
   fprintf(stderr,  "         -s <socket> : change socketname (default " DEFSOCKETNAME ")\n");
   fprintf(stderr,  "         -u <user>   : change MySQL userid   \n");
   fprintf(stderr,  "         -k <passwd> : supply MySQL password \n");
   fprintf(stderr,  "         -c          : disable record count display\n");
   fprintf(stderr,  "         -n          : disable confirmation of record actions\n");
   exit(1);
}

/* Attach the widgets we need in the code to the Params structure */
void storeWidgets(GtkWindow *main)
{
   Params *p = getParams();
   
   p->querywindow  = GTK_WIDGET(lookup_widget(main, "querywindow"));
   p->logwindow    = GTK_WIDGET(lookup_widget(main, "logwindow"));
   
   p->notebook     = GTK_WIDGET(lookup_widget(main, "notebook"));

   p->dbaselist    = GTK_OPTION_MENU(lookup_widget(main, "dbaselist"));
   p->tablelist    = GTK_OPTION_MENU(lookup_widget(main, "tablelist"));

   p->logbuffer    = gtk_text_view_get_buffer (GTK_TEXT_VIEW (p->logwindow));
   p->querybuffer  = gtk_text_view_get_buffer (GTK_TEXT_VIEW (p->querywindow));
   
   p->tabinfo[RECORDSTAB].view = GTK_TREE_VIEW(lookup_widget(main, "recordview"));
   p->tabinfo[FIELDSTAB].view  = GTK_TREE_VIEW(lookup_widget(main, "fieldview"));
   p->tabinfo[SEARCHTAB].view  = GTK_TREE_VIEW(lookup_widget(main, "searchview"));
   p->tabinfo[QUERYTAB].view   = GTK_TREE_VIEW(lookup_widget(main, "queryview"));

   setLogWindow() ; /* avoids passing logwindow handles to all dialogs */
}

void gequelDestroy(GtkWidget *widget, gpointer user_data)
{
   ldebug(2,"gequelDestroy called: exiting ...\n");
   gtk_main_quit();
}

void connectMainWindowSignals(GtkWindow *main)
{
   glong x;
   Params *p = getParams();
   
   ldebug(2,"connectMainWindowSignals entered\n");

   /* Menus : */
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_loadquery")), "activate",
		       GTK_SIGNAL_FUNC (on_file_loadquery_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_savequery")), "activate",
		       GTK_SIGNAL_FUNC (on_file_savequery_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_saverecords")), "activate",
		       GTK_SIGNAL_FUNC (on_file_saverecords_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_savelog")), "activate",
		       GTK_SIGNAL_FUNC (on_file_savelog_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_print")), "activate",
		       GTK_SIGNAL_FUNC (on_file_print_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"file_exit")), "activate",
		       GTK_SIGNAL_FUNC (on_file_exit_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"edit_delete")), "activate",
		       GTK_SIGNAL_FUNC (on_edit_delete_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"edit_modify")), "activate",
		       GTK_SIGNAL_FUNC (on_edit_modify_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"edit_insert")), "activate",
		       GTK_SIGNAL_FUNC (on_edit_insert_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"edit_copy")), "activate",
		       GTK_SIGNAL_FUNC (on_edit_copy_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"settings_confirm_actions")), "activate",
		       GTK_SIGNAL_FUNC (on_settings_confirm_actions_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"settings_show_grid")), "activate",
		       GTK_SIGNAL_FUNC (on_settings_show_grid_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"help_contents")), "activate",
		       GTK_SIGNAL_FUNC (on_help_contents_activate), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"help_about")), "activate",
		       GTK_SIGNAL_FUNC (on_help_about_activate), NULL);
   
   /* Buttons : */
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"showfields")), "clicked",
		       GTK_SIGNAL_FUNC (on_showfields_clicked), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"showrecords")), "clicked",
		       GTK_SIGNAL_FUNC (on_showrecords_clicked), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"insert")), "clicked",
		       GTK_SIGNAL_FUNC (on_insert_clicked), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"delete")), "clicked",
		       GTK_SIGNAL_FUNC (on_delete_clicked), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"modify")), "clicked",
		       GTK_SIGNAL_FUNC (on_modify_clicked), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"copy")), "clicked",
		       GTK_SIGNAL_FUNC (on_copy_clicked), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"search")), "clicked",
		       GTK_SIGNAL_FUNC (on_search_clicked), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"createtable")), "clicked",
		       GTK_SIGNAL_FUNC (on_createtable_clicked), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"edittable")), "clicked",
		       GTK_SIGNAL_FUNC (on_edittable_clicked), NULL);
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"querybutton")), "clicked",
                      GTK_SIGNAL_FUNC (on_query_clicked), NULL);
   
   /* Textentries : */
   
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"server")), "activate",
                      GTK_SIGNAL_FUNC (on_server_activate), NULL);
   
   /* Tabs : */
   
   gtk_signal_connect (GTK_OBJECT (lookup_widget (main,"notebook")), "switch-page",
		       GTK_SIGNAL_FUNC (on_notebooktab_switched), NULL);

   /* connect callbacks for a singleclick on a row in each of the tab views
    * (store rownr in tabinfo) :
    */
   for(x=0;x<p->nrtabs;x++)
   {
      GtkTreeSelection *select;
            
      select = gtk_tree_view_get_selection (p->tabinfo[x].view);
      gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
      g_signal_connect (G_OBJECT (select), "changed",
			G_CALLBACK (view_selected), (gpointer) x);
   }

   /* connect callbacks for a doubleclick on a row in each of the tab views
    * (open recordwindow or change the sortkey) :
    */
   
   g_signal_connect (G_OBJECT (lookup_widget (main,"recordview")),
		     "row-activated", G_CALLBACK (record_doubleclick), NULL);
   g_signal_connect (G_OBJECT (lookup_widget (main,"fieldview")),
		     "row-activated", G_CALLBACK (field_doubleclick), NULL);
   g_signal_connect (G_OBJECT (lookup_widget (main,"searchview")),
		     "row-activated", G_CALLBACK (srecord_doubleclick), NULL);
   g_signal_connect (G_OBJECT (lookup_widget (main,"queryview")),
		     "row-activated", G_CALLBACK (qrecord_doubleclick), NULL);
   
   /* Main window */
   
   gtk_signal_connect(GTK_OBJECT (main), "destroy",
		    GTK_SIGNAL_FUNC (gequelDestroy), NULL);
}

void setStyles()
{
   setBoldFont (lookup_widget(getTopWindow(),"titlelabel"));
   setBoldFont (lookup_widget(getTopWindow(),"serverlabel"));
   setBoldFont (lookup_widget(getTopWindow(),"dbaselabel"));
   setBoldFont (lookup_widget(getTopWindow(),"tablelabel"));
}

void gequelInit(int argc, char *argv[])
{
   int x;
   char *basename;
   Params *p = getParams();

   /* WATCHOUT : GTK parses arguments first and may have removed those that are
    * recognized by GTKlib itself before we enter this routine !!!! */

   ldebug(2,"gequelInit called\n");
   basename=argv[0];
   p->countflag = COUNT;
   p->nrtabs = TABCOUNT;

   // make all notebooktabs inactive and set initial refresh flag to TRUE :
   for(x=0; x< TABCOUNT; x++)
   {
      p->tabinfo[x].inuse   = FALSE;
      p->tabinfo[x].refresh = TRUE;
   }
   
   setStyles();
   
   x=argc;
   while(--x>0)                    /* parse options */
   {
      if(*argv[1]=='-')
      {
	 switch(*(++argv[1]))
	 {
	  case 'c': p->countflag = NOCOUNT;             break;
	  case 'h': if(argc <= 2) usage(basename);
	    p->server=argv[2];     ++argv; --argc; --x; break;
	  case 's': if(argc <= 2) usage(basename);
	    p->socketname=argv[2]; ++argv; --argc; --x; break;
	  case 'u': if(argc <= 2) usage(basename);
	    p->user=argv[2];       ++argv; --argc; --x; break;
	  case 'k': if(argc <= 2) usage(basename);
	    p->pwd=argv[2];        ++argv; --argc; --x; break;
	  case 'n': gtk_check_menu_item_set_active
	      (GTK_CHECK_MENU_ITEM(lookup_widget (getTopWindow(),"settings_confirm_actions")),0);
	    break;
	  default:  usage(basename);      break;
	 }
	 ++argv; --argc;
      }
   }
   
   if(argc > 3)
     usage(basename);
   
   setNewServer(p->server,p->socketname);
   
   if(argc >= 2)
   {
      setDBaseList(argv[1]);
      setNewDatabase(argv[1]);
   }
   
   if(argc == 3)
   {
      setTableList(argv[2]);
      setNewTable(argv[2]);
   }

}
